@@ -1,5 +1,9 @@
+# from __future__ import annotations
 from copy import deepcopy
-from dataclasses import InitVar, asdict, dataclass, field
+from dataclasses import InitVar, asdict, dataclass, field, astuple
+
+from tesla.pyhtml.tags import CT, CSS
+
 
 from datetime import datetime
 # from tempfile import TemporaryFile
@@ -11,10 +15,11 @@ from tesla.request import TemporaryFile
 _no_default_object = object()
 def property_obj(arr):
     a = []
-    for l in arr:
+    for l in arr.__dict__:
         if not l.startswith('__'):
-            a.append(l)
-            
+            if l != 'id' and l != 'timestamp':
+                a.append(l)
+
     return a        
 
 def ModalId():
@@ -41,75 +46,132 @@ class ModalObject:
 
     def get(self, **kwargs):
         return self.obj.get(**kwargs)
-        
 
-
-@dataclass
-class bModel:
-    id : str
-    timestamp : str
-
-    @classmethod
-    def get(cls,  **kwargs):
-        # print(dir(cls))
-        db = JsonDB(collection = cls.__name__ + "/") 
-        js = db.get(kwargs)
-        # print(js)
-        
-        return type(cls.__name__, (), js)
-        # for k, v in js.items():
-        #     setattr(cls, k, v)
-        print(cls.__init__(**js))
-        return cls.__init__(cls, **js)
-
-    @classmethod
-    def filter(cls, **kwargs):
-        # print(dir(cls))
-        db = JsonDB(collection = cls.__name__ + "/") 
-        return db.filter(kwargs)   
-
-    @classmethod
-    def all(cls, **kwargs):
-        # print(dir(cls))
-        db = JsonDB(collection = cls.__name__ + "/") 
-        return db.all()       
+class Field:
     
-    @property
-    def db(self):
-        return JsonDB(collection = self.modal_name() + "/") 
-
-    def modal_name(self):
-        return (self.__class__.__name__)
-
-
-
-    def save(self):
-        js = self.json() 
-        print(js)
-        for k, v in js.items():
-            if isinstance(v, TemporaryFile):
-                # print(v.save())
-                js[k] = v.save() 
-        if not hasattr(self, 'id'):
-            self.id = str(uuid.uuid4())
-            js['id'] = self.id
-        js['timestamp'] = str(datetime.now())
-        self.db.create_column(model = js, table_name = str(self.id))
+    def __init__(self, required = False, label = None) -> None:
+        self.input_type = 'text'
+        self.type = 'input'
+        self.required = required
+        self.css = CSS()
+        self.tag = None
+        self.name = ''
+        self.params = ''
+        self.label = None
+        self.value_type = str
+        pass        
+    
+    def input(self, name, **kwargs):
+        k = ' '.join([f'{a}={b}' for a,b in kwargs.items()]) 
+        self.params += ' ' + k
+        self.name = name
+        if self.required:
+            self.params += ' required'
+        if self.type == 'textarea':
+            self.css.kwargs['height'] = '200px'      # s
+            self.css.kwargs['resize'] = 'none'      # s
         return self
     
-    def json(self):
-        return self.__dict__
+
+    def html(self) -> str:
+        self.input(self.name)
+        self.tag = CT(self.type , **{'class':"form-control"},  style=self.css.css(), type=self.input_type, name=self.name, value=self.default, params=self.params)
+        # self.tag = CT(self.type, value=self.default)
+        if self.type != 'input':
+            self.tag.append(self.default)
+            del self.tag.kwargs['value']
+        if self.label == None:
+            self.label = self.name  
+        # print(self.tag.html())      
+        return self.tag.html()
+
+    def __str__(self):
+        self.html()
+        return self.tag.html()  
+
+    def __repr__(self):
+        return self.__class__.__name__
+    def validate(self, value):
+        try:
+            v = self.value_type(value)
+            if isinstance(v, self.value_type):
+                return True
+            return False 
+        except Exception:
+            raise Exception(f'{value} is not valid type for {self.__repr__()}')
+
+               
+class CharField(Field):
+    
+    def __init__(self, default='', min=0, max=120, *args, **kwargs) -> None:
+        super().__init__( *args, **kwargs)
+        self.default = default
+        self.min = min
+        self.max = max
+
+        
+    
+    def input(self, *args):
+        kwargs = {}
+        kwargs['minlength'] = self.min 
+        kwargs['maxlength'] = self.max 
+        return super().input(*args, **kwargs)    
+        
+class PasswordField(CharField):
+    
+    def __init__(self , *args, **kwargs) -> None:
+        super().__init__( *args, **kwargs)
+        self.input_type = 'password' 
+       
+    
+    
+
+
+class ListField(Field, list):
+    
+    def __init__(self, default=[], *args, **kwargs) -> None:
+        super().__init__( *args, **kwargs)
+        self.default = default
+        self.type = 'select'
+        
+class TextField(Field):
+    
+    def __init__(self,default='', *args, **kwargs) -> None:
+        super().__init__( *args, **kwargs)
+        self.default = default
+        self.type = 'textarea'        
+
+class EmailField(Field):
+    
+    def __init__(self, default='', *args, **kwargs) -> None:
+        super().__init__( *args, **kwargs)
+        self.default = default
+        self.input_type = 'email'
+
+class DateField(CharField):
+    
+    def __init__(self, *args, **kwargs) -> None:
+        super().__init__( *args, **kwargs)
+        self.input_type = 'date'
+
+class DictField(Field, dict):
+    
+    def __init__(self, default={}, *args, **kwargs) -> None:
+        super().__init__( *args, **kwargs)
+        self.default = default
+
 
 
-@dataclass
 class Model:
     
-    id : str 
-    timestamp : str 
+    id = CharField() 
+    timestamp = CharField()
+    
+    
     
     def __init__(self) -> None:
         self.assign_foreign()
-        print('here')
+        
         # pass
     
     @classmethod
@@ -121,7 +183,9 @@ class Model:
     def create(cls, *args, **kwargs):
         kwargs['id'] = str(uuid.uuid4())
         kwargs['timestamp'] = str(datetime.now())
-        c = cls(**kwargs)
+        c = cls()
+        for k, v in kwargs.items():
+            setattr(c, k, v)
         # c.assign_foreign()
         return c
     
@@ -147,14 +211,15 @@ class Model:
         # print(kwargs)
         db = JsonDB(collection = cls.__name__ + "/") 
         js = db.get(kwargs)
+        # print(js)
         c = None
         if js != None:
-            c = cls(**js)
+            c = cls()
+            for k, v in js.items():
+                setattr(c, k, v)
         # c.assign_foreign()
         return c
-        pass
-    
-    
+        
     @classmethod
     def __subclasses(cls):
         l = cls.__subclasses__()
@@ -187,6 +252,10 @@ class Model:
         # self.db.g    
         return self
     
+    def update(self, **kwargs):
+        for k, v in kwargs.items():
+            setattr(self ,k , v)
+        return self    
     def json(self):
         return self.__dict__
     
@@ -218,12 +287,27 @@ class Model:
     def __meta__(cls):
         return ('id',)
     
-    @property
+    @classmethod
+    def property_cls(cls):
+        ls = property_obj(cls)
+       
+        # print(cls.__dict__, cls.__flags__)
+        # ins = cls()
+        # print(ins.id)
+        # for i, f in enumerate(ls):
+        # #   print(f)
+        #   ls[i] = f
+            # ls.append(f)
+            
+        return ls
+    
+    # @property
     def admin_dis(self):
         tp = self.__meta__()
         f = []
         for t in tp:
             f.append(getattr(self, t))
-        print(f)    
+         
         return f    
+
     
\ No newline at end of file
