@@ -52,7 +52,7 @@ class Render(Response):
             # template = env.get_template(self.is_available(content))
         file_loader = FileSystemLoader(self.templates_folders)
         loader = ChoiceLoader([
-    PackageLoader("tesla.admin", 'templates'),
+    # PackageLoader("tesla.admin", 'templates'),
     FileSystemLoader(self.templates_folders)
 ])
         # env = Environment(loader=file_loader)
@@ -128,7 +128,7 @@ class Http500Response(ErrorResponse):
 
 
 class Redirect(Response):
-    def __init__(self, request: Request, route,  status_code='302 Found', content_type='text/html'):
+    def __init__(self, request: Request, route,  status_code='302 Found', content_type='text/html', *args, **kwargs):
         super().__init__(request, status_code, content_type)
-        self.headers += [('Location', tesla.functions.url(route))]
+        self.headers += [('Location', tesla.functions.url(route, **kwargs))]
         # print(self.headers)
